//====================================================== IBJpluris.h
//
// Header for Plume Rise Model PLURIS
// ==================================
//
// Copyright (C) Janicke Consulting, Dunum, Germany, 1998-2002
// email: info@janicke.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// history:
//
// 2000-05-12 uj added mass concept etc. (used for publication)
//
//==================================================================

#define ALPHA4     0.38
#define ALPHA1     0.15
#define SIGC       3.
#define ALPHA2     0.6
#define PIFAC      3.141592654
#define PRESSURE   101300.
#define LHEAT      2454300.
#define RVAPOUR    461.53
#define RAIR       287.06
#define TZERO      273.16
#define NEQ        13
#define CPRESS     1004.1
#define GRAV       9.8066
#define LOCP       2444.2785
#define MINVAL     1.e-8
#define ZMAX       1100.0

////// plume rise output
typedef struct {
  double ll; double hh; double hf; double vdi; double lst;
  } PLROUT0;
PLROUT0 *Out0;


////// minimale Ausgabe
typedef struct {
  double xx; double yy; double zz; double hh; double ss;
  double rr; double aa; double uu; double jj; double tt; double cc; double tm;
  } PLROUT1;
PLROUT1 *Out1;

////// maximale Ausgabe (fr JAVA u..)
typedef struct {
  double xx; double yy; double zz; double hh; double ss;
  double ll; double ww;
  double rr; double uu; double jj; double tt; double cc;
  double dd; double ee; double w1; double w2;
  double aa; double qi; double li; double zi;
  double rc; double c0; double c1;
  double ls; double ws; double zs; double hs;
  double w1r; double z1r; double h1r;
  double w2r; double z2r; double h2r;
  double w1a; double z1a; double h1a;
  double w2a; double z2a; double h2a;
  } PLROUT2;
PLROUT2 *Out2;

////// Ausgabe zur Konturzeichnung (unskaliert (3) und unskaliert (4))
typedef struct {
  double xx; double yy; double zz; double hh; double ss;
  double ll; double ww; double rr; double aa;
  double w1r; double z1r; double h1r;
  double w2r; double z2r; double h2r;
  double w1a; double z1a; double h1a;
  double w2a; double z2a; double h2a;
  } PLROUT3;
PLROUT3 *Out3;

typedef struct {
  double ll; double hh; 
  double bm; double bb;
  } PLROUT5;
PLROUT5 *Out5;


#define OUT0(a)  (*(PLROUT0*)AryPtrX(&PlrOut,(a)))
#define OUT1(a)  (*(PLROUT1*)AryPtrX(&PlrOut,(a)))
#define OUT2(a)  (*(PLROUT2*)AryPtrX(&PlrOut,(a)))
#define OUT3(a)  (*(PLROUT3*)AryPtrX(&PlrOut,(a)))
#define OUT5(a)  (*(PLROUT5*)AryPtrX(&PlrOut,(a)))

typedef struct {
  float ha, ua, da, ia, ta, ra, ca;
} AKTREC;


ARYDSC  PlrOut;

static char *eMODn = "IBJpluris";

FILE *InpFile, *PrfFile, *OutFile;
char Dir[512];
char InpName[512];
char PrfName[512];
char OutName[512];
char LogName[512];
char PlrOutFormat[1000];


