// Copyright (C) Janicke Consulting, Dunum, Germany, 1998-2001
// email: info@janicke.de

// This is the header IBJtab.h to be used by the program "IBJpluris".
// It defines all parameters whose values can be given by the user.
// It has to be included by all modules that use these values.

// Each parameter is defined in a record of the struct PrmTab.
// The name of the record is the same as the name of the input
// parameter (small letters). The record contains the name, a
// short description, input and output format and the default
// value as a string. For arrays also the number of elements or
// a reference to a parameter is given, that represents this
// number. For details see the header IBJprm.h.

#ifndef  IBJPRM_INCLUDE
    #include "IBJprm.h"  // The typedef of PRMTABREC is needed
#endif

#ifndef  IBJPRM_SOURCE
    extern  // Only IBJPRM saves the data of this structure
#endif
struct { PRMTABREC  // The names of the input parameters:
 _q, a1, a2, cq, dq, fq, hq, id, iq, sq, rq, lq, tq, qq, uq, xq, yq, _a, ca, da,
 ef, ta, ua, lm, km, ve, ha, hm, z0, d0, us, tx, th, ux, uh, ia, nz, pa, ra, vs,
 za, _c, oo, sc, sd, se, sn, _e, __, ds, te, dp, dx, dy, dz, ff, zv, uv, dv, xv,
 yv, wv, iv, jv, tv, pv, rv, qv, cv, sv, xx, yy, zz, hh, ss, ll, ww, zt, kk, uu,
 jj, tt, dd, cc, qi, li, zi, rr, aa, w1, w2, ee, hu, kl, ul, jl, tl, dl, cl, ql,
 sl, zl, pl, end_of_table; } PrmTab
#ifdef  IBJPRM_SOURCE
//      The initial values are defined in IBJPRM
 = {
 { "*q",  "%s",  " %255s",       "",  "0", 255,  0, 0x005, 0, "Angaben zur Quelle" },
 { "a1", "%lf", " %12.4lf",     "90",  "0",   8,  0, 0x004, 0, "Austrittswinkel gegen die Horizontale (Grad)" },
 { "a2", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Austrittswinkel gegen x-Achse gegen Uhrzeigersinn (Grad)" },
 { "cq", "%lf", " %12.4le",      "1",  "0",   8,  0, 0x004, 0, "Anfangskonzentration (ME/m3)" },
 { "dq", "%lf", " %12.4lf",      "1",  "0",   8,  0, 0x004, 0, "Durchmesser der Quelle (m)" },
 { "fq", "%lf", " %12.4lf",     "-1",  "0",   8,  0, 0x004, 0, "Quadrat der Froudezahl beim Quellaustritt (1)" },
 { "hq", "%lf", " %12.4lf",     "10",  "0",   8,  0, 0x004, 0, "Hhe der Quelle (m)" },
 { "id",  "%s",  " %-50s",  "keine",  "0",  50,  0, 0x005, 0, "Kurz-Information" },
 { "iq", "%lf", " %12.4lf",    "0.1",  "0",   8,  0, 0x004, 0, "Turbulenzintensitt (1)" },
 { "sq", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Spezifische Feuchte (kg/kg)" },
 { "rq", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Relative Feuchte (1)" },
 { "lq", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Spezifischer Flssigwassergehalt (kg/kg)" },
 { "tq", "%lf", " %12.4lf",     "20",  "0",   8,  0, 0x004, 0, "Temperatur der Quelle (C)" },
 { "qq", "%lf", " %12.4lf",     "-1",  "0",   8,  0, 0x004, 0, "Wrmestrom der Quelle (MW)" },
 { "uq", "%lf", " %12.4lf",     "10",  "0",   8,  0, 0x004, 0, "Austrittsgeschwindigkeit (m/s)" },
 { "xq", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "X-Koordinate der Quelle/Mittelpunkt (m)" },
 { "yq", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Y-Koordinate der Quelle/Mittelpunkt (m)" },
 { "*a",  "%s",  " %255s",       "",  "0", 255,  0, 0x005, 0, "Angaben zur Umgebung" },
 { "ca", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Konzentration in der Umgebung (ME/m3)" },
 { "da", "%lf", " %12.4lf",    "270",  "0",   8,  0, 0x004, 0, "Windrichtung gegen Nord im Uhrzeigersinn (Grad)" },
 { "ef", "%lf", " %12.6lf",      "1",  "0",   8,  0, 0x004, 0, "Reduktionsfaktor fr die Entrainment-Funktion" },
 { "ta", "%lf", " %12.4lf",     "10",  "0",   8,  0, 0x004, 0, "Temperatur in Anemometerhhe (C)" },
 { "ua", "%lf", " %12.4lf",      "3",  "0",   8,  0, 0x004, 0, "Geschwindigkeit in Anemometerhhe (m/s)" },
 { "lm", "%lf", " %12.3lf",      "0",  "0",   8,  0, 0x004, 0, "Monin-Obukhov-Lnge (m)" },
 { "km",  "%d",   " %12d",      "0",  "0",   4,  0, 0x002, 0, "Klug/Manier-Klasse (1..6)" },
 { "ve",  "%d", " %12.0d",      "2",  "0",   4,  0, 0x002, 0, "Profil-Version fr IBJprf (nur fr lm!=0)" },
 { "ha", "%lf", " %12.6lf",     "10",  "0",   8,  0, 0x004, 0, "Anemometerhhe (nur fr lm!=0) (m)" },
 { "hm", "%lf", " %12.3lf",      "0",  "0",   8,  0, 0x004, 0, "Mischungsschichthhe (nur fr lm!=0) (m)" },
 { "z0", "%lf", " %12.6lf",    "1.0",  "0",   8,  0, 0x004, 0, "Rauhigkeitslnge (nur fr lm!=0) (m)" },
 { "d0", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Verdrngungshhe (nur fr lm!=0) (m)" },
 { "us", "%lf", " %12.6lf",      "0",  "0",   8,  0, 0x004, 0, "Schubspannungsgeschwindigkeit (nur fr lm!=0) (m/s)" },
 { "tx", "%lf", " %12.4lf", "-0.0098",  "0",   8,  0, 0x004, 0, "Temperaturgradient (nur fr lm=0)" },
 { "th", "%lf", " %12.4lf",      "2",  "0",   8,  0, 0x004, 0, "Anemometerhhe der Temperatur (nur fr lm=0) (m)" },
 { "ux", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Geschwindigkeitsexponent (nur fr lm=0)" },
 { "uh", "%lf", " %12.4lf",     "10",  "0",   8,  0, 0x004, 0, "Anemometerhhe der Geschwindigkeit (nur fr lm=0) (m)" },
 { "ia", "%lf", " %12.4lf",    "0.1",  "0",   8,  0, 0x004, 0, "Turbulenzintensitt (1)" },
 { "nz",  "%d", " %12.0d",    "250",  "0",   4,  0, 0x002, 0, "Anzahl der vertikalen Sttzpunkte" },
 { "pa", "%lf", " %12.4le", "101300",  "0",   8,  0, 0x004, 0, "Standarddruck in der untersten Mehhe (Pa)" },
 { "ra", "%lf", " %12.4le",   "0.77",  "0",   8,  0, 0x004, 0, "Relative Feuchte (1)" },
 { "vs", "%lf", " %12.6lf",      "0",  "0",   8,  0, 0x004, 0, "Sedimentationsgeschwindigkeit (m/s)" },
 { "za", "%lf", " %12.4lf",     "10",  "0",   8,  0, 0x004, 0, "Vertikale Schrittweite zur Erzeugung der Profile (m)" },
 { "*c",  "%s",  " %255s",       "",  "0", 255,  0, 0x005, 0, "Angaben zur Rechnung" },
 { "oo",  "%d", " %12.0d",      "2",  "0",   4,  0, 0x002, 0, "Ausgabemodus (1:wenig/2:viel/3:Rnder/4:3 skaliert)" },
 { "sc", "%lf", " %12.4lf",      "1",  "0",   8,  0, 0x004, 0, "Skalierung (m)" },
 { "sd", "%lf", " %12.4lf",   "0.01",  "0",   8,  0, 0x004, 0, "Skalierte Rechen-Schrittweite (sc)" },
 { "se", "%lf", " %12.4lf",    "100",  "0",   8,  0, 0x004, 0, "Skalierter Endpunkt (sc)" },
 { "sn",  "%d", " %12.0d",    "100",  "0",   4,  0, 0x002, 0, "Anzahl der Ausgabeschritte" },
 { "*e",  "%s",  " %255s",       "",  "0", 255,  0, 0x005, 0, "Ende der Rechnung" },
 { "*_",  "%s",  " %255s",       "",  "0", 255,  0, 0x005, 0, "Interne Parameter (knnen nicht vorgegeben werden)" },
 { "ds", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Rechen-Schrittweite (m)" },
 { "te", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Ende (m)" },
 { "dp", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Ausgabe-Schrittweite (m)" },
 { "dx", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Schrittweite (m)" },
 { "dy", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Schrittweite (m)" },
 { "dz", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Schrittweite (m)" },
 { "ff", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Variable zur Ausgabe der Fahnenmantel-Werte" },
 { "zv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - z (m)" },
 { "uv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - Windgeschw. (m/s)" },
 { "dv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - Winkel gegen Nord (Grad)" },
 { "xv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - vx (m/s)" },
 { "yv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - vy (m/s)" },
 { "wv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - vz (m/s)" },
 { "iv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - Turbulenzint. (1)" },
 { "jv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - Windfluktuationen (m/s)" },
 { "tv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - Temperatur (K)" },
 { "pv", "%lf", " %12.4le",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - Druck (Pa)" },
 { "rv", "%lf", " %12.4le",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - rel. Feuchte (1)" },
 { "qv", "%lf", " %12.4le",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - spez. Feuchte (kg/kg)" },
 { "cv", "%lf", " %12.4le",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - Konzentration (ME/m3)" },
 { "sv", "%lf", " %12.4lf",      "0", "nz+1",   8, -1, 0x104, 0, "Vertikales Profil der Umgebung - SigmaW (m/s)" },
 { "xx", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "X-Koordinate (m)" },
 { "yy", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Y-Koordinate (m)" },
 { "zz", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Z-Koordinate (m)" },
 { "hh", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "H-Koordinate (m) (Quellberhhung zz-hq)" },
 { "ss", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "S-Koordinate (Weg entlang der Fahnenachse) (m)" },
 { "ll", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "L-Koordinate (Weg entlang der horizontalen Projektion) (m)" },
 { "ww", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "W-Koordinate (horizontaler Abstand von der Quelle) (m)" },
 { "zt", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Transportzeit (s)" },
 { "kk", "%lf", " %12.4lf",      "0",  "4",   8,  4, 0x104, 0, "Kartesische Geschwindigkeiten der Fahne (m/s)" },
 { "uu", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Mittlere Geschwindigkeit der Fahne (m/s)" },
 { "jj", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Turbulente Geschwindigkeit der Fahne (m/s)" },
 { "tt", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Temperatur der Fahne (K)" },
 { "dd", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Dichte der Fahne (kg/m3)" },
 { "cc", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Konzentration in der Fahne (ME/m3)" },
 { "qi", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Spezifische Feuchte der Fahne (kg/kg)" },
 { "li", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Spezifischer Flssigwassergehalt der Fahne (kg/kg)" },
 { "zi", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Spezifischer Gesamtwassergehalt der Fahne (kg/kg)" },
 { "rr", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Radius der Fahne (m)" },
 { "aa", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Sichtbarer Radius der Fahne (m)" },
 { "w1", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Winkel zur Horizontalen (Rad)" },
 { "w2", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Winkel gegen Nord (Rad)" },
 { "ee", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Entrainment (m2/s)" },
 { "hu", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Geschwindigkeit in Quellhhe (m)" },
 { "kl", "%lf", " %12.4lf",      "0",  "4",   8,  4, 0x104, 0, "Kartesische Geschwindigkeiten der Umgebung (m/s)" },
 { "ul", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Mittlere Geschwindigkeit der Umgebung (m/s)" },
 { "jl", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Turbulente Geschwindigkeit der Umgebung (m/s)" },
 { "tl", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Temperatur der Umgebung (K)" },
 { "dl", "%lf", " %12.4lf",      "0",  "0",   8,  0, 0x004, 0, "Dichte der Umgebung (kg/m3)" },
 { "cl", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Konzentration in der Umgebung (ME/m3)" },
 { "ql", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Spezifische Feuchte der Umgebung (kg/kg)" },
 { "sl", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Spezifischer Flssigwassergehalt der Umgebung (kg/kg)" },
 { "zl", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Spezifischer Gesamtwassergehalt der Umgebung (kg/kg)" },
 { "pl", "%lf", " %12.4le",      "0",  "0",   8,  0, 0x004, 0, "Druck in der Umgebung (Pa)" },
 { "**", NULL, NULL, NULL, NULL, 0, 0, 0, NULL } }
#endif
;
//  Macros for easier reference:

#define  A1  PrmTab.a1.u.d
#define  A2  PrmTab.a2.u.d
#define  CQ  PrmTab.cq.u.d
#define  DQ  PrmTab.dq.u.d
#define  FQ  PrmTab.fq.u.d
#define  HQ  PrmTab.hq.u.d
#define  ID  (char*)PrmTab.id.u.p
#define  IQ  PrmTab.iq.u.d
#define  SQ  PrmTab.sq.u.d
#define  RQ  PrmTab.rq.u.d
#define  LQ  PrmTab.lq.u.d
#define  TQ  PrmTab.tq.u.d
#define  QQ  PrmTab.qq.u.d
#define  UQ  PrmTab.uq.u.d
#define  XQ  PrmTab.xq.u.d
#define  YQ  PrmTab.yq.u.d
#define  CA  PrmTab.ca.u.d
#define  DA  PrmTab.da.u.d
#define  EF  PrmTab.ef.u.d
#define  TA  PrmTab.ta.u.d
#define  UA  PrmTab.ua.u.d
#define  LM  PrmTab.lm.u.d
#define  KM  PrmTab.km.u.i
#define  VE  PrmTab.ve.u.i
#define  HA  PrmTab.ha.u.d
#define  HM  PrmTab.hm.u.d
#define  Z0  PrmTab.z0.u.d
#define  D0  PrmTab.d0.u.d
#define  US  PrmTab.us.u.d
#define  TX  PrmTab.tx.u.d
#define  TH  PrmTab.th.u.d
#define  UX  PrmTab.ux.u.d
#define  UH  PrmTab.uh.u.d
#define  IA  PrmTab.ia.u.d
#define  NZ  PrmTab.nz.u.i
#define  PA  PrmTab.pa.u.d
#define  RA  PrmTab.ra.u.d
#define  VS  PrmTab.vs.u.d
#define  ZA  PrmTab.za.u.d
#define  OO  PrmTab.oo.u.i
#define  SC  PrmTab.sc.u.d
#define  SD  PrmTab.sd.u.d
#define  SE  PrmTab.se.u.d
#define  SN  PrmTab.sn.u.i
#define  DS  PrmTab.ds.u.d
#define  TE  PrmTab.te.u.d
#define  DP  PrmTab.dp.u.d
#define  DX  PrmTab.dx.u.d
#define  DY  PrmTab.dy.u.d
#define  DZ  PrmTab.dz.u.d
#define  FF  PrmTab.ff.u.d
#define ZV(i) ((double*)PrmTab.zv.u.p)[i]
#define UV(i) ((double*)PrmTab.uv.u.p)[i]
#define DV(i) ((double*)PrmTab.dv.u.p)[i]
#define XV(i) ((double*)PrmTab.xv.u.p)[i]
#define YV(i) ((double*)PrmTab.yv.u.p)[i]
#define WV(i) ((double*)PrmTab.wv.u.p)[i]
#define IV(i) ((double*)PrmTab.iv.u.p)[i]
#define JV(i) ((double*)PrmTab.jv.u.p)[i]
#define TV(i) ((double*)PrmTab.tv.u.p)[i]
#define PV(i) ((double*)PrmTab.pv.u.p)[i]
#define RV(i) ((double*)PrmTab.rv.u.p)[i]
#define QV(i) ((double*)PrmTab.qv.u.p)[i]
#define CV(i) ((double*)PrmTab.cv.u.p)[i]
#define SV(i) ((double*)PrmTab.sv.u.p)[i]
#define  XX  PrmTab.xx.u.d
#define  YY  PrmTab.yy.u.d
#define  ZZ  PrmTab.zz.u.d
#define  HH  PrmTab.hh.u.d
#define  SS  PrmTab.ss.u.d
#define  LL  PrmTab.ll.u.d
#define  WW  PrmTab.ww.u.d
#define  ZT  PrmTab.zt.u.d
#define KK(i) ((double*)PrmTab.kk.u.p)[i]
#define  UU  PrmTab.uu.u.d
#define  JJ  PrmTab.jj.u.d
#define  TT  PrmTab.tt.u.d
#define  DD  PrmTab.dd.u.d
#define  CC  PrmTab.cc.u.d
#define  QI  PrmTab.qi.u.d
#define  LI  PrmTab.li.u.d
#define  ZI  PrmTab.zi.u.d
#define  RR  PrmTab.rr.u.d
#define  AA  PrmTab.aa.u.d
#define  W1  PrmTab.w1.u.d
#define  W2  PrmTab.w2.u.d
#define  EE  PrmTab.ee.u.d
#define  HU  PrmTab.hu.u.d
#define KL(i) ((double*)PrmTab.kl.u.p)[i]
#define  UL  PrmTab.ul.u.d
#define  JL  PrmTab.jl.u.d
#define  TL  PrmTab.tl.u.d
#define  DL  PrmTab.dl.u.d
#define  CL  PrmTab.cl.u.d
#define  QL  PrmTab.ql.u.d
#define  SL  PrmTab.sl.u.d
#define  ZL  PrmTab.zl.u.d
#define  PL  PrmTab.pl.u.d
