//======================================================== IBJmsg.c
//
// Message Routines for IBJ-programs
// =================================
//
// Copyright (C) Umweltbundesamt, 14191 Berlin, Germany, 2002
// Copyright (C) Janicke Consulting, 26427 Dunum, Germany, 2002
// email: info@austal2000.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// last change 2004-08-20 uj
//
//==================================================================

char *IBJmsgVersion = "1.2.11";
static char *eMODn = "IBJmsg";

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#include "IBJmsg.h"

FILE *MsgFile;
int MsgCode, MsgBreak, MsgLevel=0, MsgQuiet=0, MsgVerbose=1;
int MsgLogLevel=3, MsgDspLevel=2;
int MsgTranslate=0;                               //-2002-07-03
int MsgDateHour, MsgDateMinute, MsgDateSecond;
int MsgDateYear, MsgDateMonth, MsgDateDay;
int MsgBufLen = 4000;
void *MsgPvd;
char *MsgSource;
static int StdQuiet=-99;
static int StdVerbose=-99;
static int StdLevel=0;
//
// static char *eMODn = "IBJmsg";
//
/*===================================================================== MsgLow
*/
void MsgLow(    /* convert to lower case        */
char *s )       /* string to be converted       */
  {
  while (*s) {
    *s = tolower(*s);
    s++;
    }
  }

//===================================================================== MsgTok
//
char *MsgTok(   /* non-destructive tokenisation */
char *start,    /* begin of scanning            */
char **pnext,   /* start of next scan           */
char *sep,      /* separator characters         */
int *pl )       /* number of usable characters  */
  {             /* RETURN: start of token       */
  int l, quoted;
  if ((!start) || (!sep))  return NULL;
  while (*start && strchr(sep, *start))  start++;
  if (!*start)  return NULL;
  if (*start == MsgBreak)  return NULL;
  quoted = 0;
  for (l=0; ; l++) {
    if (!start[l])  break;
    if (strchr(sep, start[l]) && !quoted)  break;
    if (start[l] == '\"') {
      if (l==0 || start[l-1]!='\\')  quoted = 1-quoted;
    }
  }
  if (pnext)  *pnext = start + l;
  if (pl)  *pl = l;
  return start;
}

//================================================================= MsgTokens
//
char **_MsgTokens(  // tokenize a string
char *t,            // string to be tokenized (not changed by this routine)
char *s )           // separator characters
{                   // RETURN: allocated array of strings, ends with NULL
  dQ(_MsgTokens);
  int n, l;
  char *pt, *pn, **_ppt;
  if (!t || !s)  return NULL;
  for (pn=t, n=0; ; n++) {
    pt = MsgTok(pn, &pn, s, &l);
    if (!pt)  break;
  }
  _ppt = (char**) ALLOC((n+1)*sizeof(char*) + strlen(t) + 1); //-2001-08-15
  if (!_ppt)  return NULL;
  pt = (char*)(_ppt+n+1);                                     //-2001-08-15
  strcpy(pt, t);                                              //-2001-08-15
  for (pn=pt, n=0; ; n++) {                                   //-2001-08-15
    pt = MsgTok(pn, &pn, s, &l);
    if (!pt)  break;
    _ppt[n] = pt;
    if (*pn)  *pn++ = 0;
  }
  _ppt[n] = NULL;
  return _ppt;
}

//================================================================ MsgDequote
//
int MsgDequote( //  remove quotation marks and escape sequences
char *s )       //  string to work on
{               //  RETURN: number of replacements
  int n;
  char *psrc, *pdst;
  if (s == NULL)  return 0;
  n = 0;
  for (psrc=s, pdst=s; (*psrc); psrc++) {
    if (*psrc == '\"') {
      n++;
      continue;
    }
    if (*psrc == '\\') {
      char c=0;
      switch (psrc[1]) {
        case '\"': c = '\"';  break;
        case '\'': c = '\'';  break;
        case '\\': c = '\\';  break;
        case '?':  c = '?';   break;
        case 'a':  c = '\a';  break;
        case 'b':  c = '\b';  break;
        case 'f':  c = '\f';  break;
        case 'n':  c = '\n';  break;
        case 't':  c = '\t';  break;
        case 'r':  c = '\r';  break;
        case 'v':  c = '\v';  break;
        default: ;
      }
      if (c) {
        *pdst++ = c;
        psrc++;
        n++;
        continue;
      }
      if (psrc[1]=='x') {
        char b[16], *tail;
        *b = 0;
        strncat(b, psrc+2, 2);
        *pdst++ = strtol(b, &tail, 16);
        psrc += 3;
        n++;
        continue;
      }
      if (isdigit(psrc[1])) {
        char b[16], *tail;
        *b = 0;
        strncat(b, psrc+1, 3);
        *pdst++ = strtol(b, &tail, 8);
        psrc += 3;
        n++;
        continue;
      }
    }
    *pdst++ = *psrc;
  }
  *pdst = 0;
  return n;
}

//================================================================ MsgUnquote
//
int MsgUnquote( //  remove quotation marks
char *s )       //  string to work on
{               //  RETURN: number of replacements
  int n;
  char *psrc, *pdst;
  if (s == NULL)  return 0;
  n = 0;
  for (psrc=s, pdst=s; (*psrc); psrc++) {
    if (*psrc == '\"') {
      n++;
      continue;
    }
    *pdst++ = *psrc;
  }
  *pdst = 0;
  return n;
}

//============================================================= MsgSetVerbose
int MsgSetVerbose( int new_value )  // set standard value of MsgVerbose
{
  int old_value = StdVerbose;
// printf("MSG set verbose=%d\n", new_value);
  MsgVerbose = new_value;
  StdVerbose = new_value;
  return old_value;
}

//============================================================= MsgSetQuiet
int MsgSetQuiet( int new_value )
{
  int old_value = StdQuiet;
// printf("MSG set quiet=%d\n", new_value);
  MsgQuiet = new_value;
  StdQuiet = new_value;
  return old_value;
}

//============================================================= MsgSetLevel
int MsgSetLevel( int new_value )
{
  int old_value = StdLevel;
  MsgLevel = new_value;
  StdLevel = new_value;
  return old_value;
}

//====================================================================== vMsg
int vMsg(       /* print message on screen and into logfile     */
char *format,   /* printing format                              */
... )           /* parameter                                    */
{
  dQ(vMsg);
  va_list argptr;
  int verbose, display;
  char time_stamp[80]="", *_form=NULL;
  static char *s, *pc;
  static char DSTcode[] = { 0x84,0x94,0x81,0x8E,0x99,0x9A,0xE6,0xE1,0xFD,0xFC,0x00 };
  static char SRCcode[] = { 0xE4,0xF6,0xFC,0xC4,0xD6,0xDC,0xB5,0xDF,0xB2,0xB3,0x00 };
  int i, l, drop_cr=0;
  if (!format) {
    if (s)  FREE(s);
    s = NULL;
    return 0;
  }
  if (!s) {
    s = ALLOC(MsgBufLen);
    if (!s) {
      printf("*** can't allocate message buffer (%d bytes)!\n", MsgBufLen);
      fflush(stdout);
      _exit(1);
    }
  }
  if (*format == '@') {
    strcpy(time_stamp, MsgDate());
    format++;
  }
  l = strlen(format);
  if (l>0 && format[l-1]=='\\') {
    drop_cr = 1;
    _form = ALLOC(l+1);                                   //-2002-06-29
    strcpy(_form, format);
    _form[l-1] = 0;
  }
  else {
    drop_cr = 0;
    _form = format;
  }
  display = (MsgQuiet < 0) ? 1 : ((MsgVerbose>MsgLevel) && !MsgQuiet); //-2001-08-10
  verbose = (MsgCode < 0) ? (MsgVerbose>=0) : (MsgVerbose>MsgLevel);
  if (display) {  //-2001-08-10
#ifdef __WATCOMC__
    int n;
    va_start(argptr, format);
    n = _vbprintf(s, MsgBufLen-1, _form, argptr);
    if (n < 0)  sprintf(s, "--- message buffer overflow!\n");
#else
    va_start(argptr, format);
    vsprintf(s, _form, argptr);        // buffer overflow not caught
#endif
    va_end(argptr);
    if (MsgTranslate) {
      l = strlen(s);
      for (i=0; i<l; i++) {
        pc = strchr(SRCcode, s[i]);
        if (pc)  s[i] = DSTcode[pc-SRCcode];
      }
    }
    if (*time_stamp)  fprintf(stdout, "%s ", time_stamp);
    else if (MsgCode < 0)  fprintf(stdout, "*** ");
    fprintf(stdout, "%s", s);
    if (!drop_cr)  fprintf(stdout, "\n");
    if (MsgSource && *MsgSource)  fprintf(stdout, "    (%s)\n", MsgSource);
    fflush(stdout);
  }
  va_start(argptr, format);
  if (MsgFile && verbose) {
    if (MsgCode < 0)  fprintf(MsgFile, "*** ");
    if (*time_stamp)  fprintf(MsgFile, "%s ", time_stamp);
    vfprintf(MsgFile, _form, argptr);
    if (!drop_cr)  fprintf(MsgFile, "\n");
    if (MsgSource && *MsgSource)  fprintf(MsgFile, "    (%s)\n", MsgSource);
    fflush(MsgFile);
  }
  va_end(argptr);
  if (drop_cr)  FREE(_form);
  MsgSource = NULL;
  MsgLevel = 0;
  if (StdQuiet != -99)    MsgQuiet = StdQuiet;
  if (StdVerbose != -99)  MsgVerbose = StdVerbose;
  if (StdLevel != -99)    MsgLevel = StdLevel;
  return MsgCode;
}

/*=============================================================== MsgDateVal
*/

static void trim( char *s )
  {
  char *p, *q;
  for (q=s; (*q<=' '); q++)  if (!*q)  break;
  for (p=s; (*q); )  *p++ = *q++;
  *p = 0;
  for (p--; p>=s; p--)
      if (*p <= ' ')  *p = 0;
      else  break;
  }

double MsgDateVal(      /* converts date string into numerical value    */
char * ss )             /* string of the form yy-mm-dd'hh:mm:ss         */
  {                     /* RETURN: days after 1899-12-30'00:00:00 +1.e6 */
  static int dayspassed[] = {
        0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365 };
  char s[80], *pc, *pd, *pt;
  int year, month, day, days;
  int hour, minute, second, n;
  double mydate;
  strcpy(s, ss);  trim(s);
  if (!strcmp(s, "-inf"))  return -9999999;
  if (!strcmp(s, "+inf"))  return 9999999;
  pc = strchr(s, ' ');
  if (!pc)  pc = strchr(s, '\'');
  if (!pc)  pc = strchr(s, '.');
  if (!pc)  pc = strchr(s, ' ');                  //-2002-02-14
  if (pc) {
      *pc = 0;
      pt = pc+1;
      pd = s; }
  else {
      if (strchr(s, '-')) {
          pd = s;
          pt = ""; }
      else {
          pd = "";
          pt = s; }
      }
  pc = strchr(pd, '-');
  mydate = 0;
  if (pc) {
      *pc++ = 0;
      year = atoi(pd);
      pd = pc;
      pc = strchr(pd, '-');
      if (pc) {
          *pc++ = 0;
          month = atoi(pd);
          day = atoi(pc); }
      else {
          month = year;
          year = 0;
          day = atoi(pd); }
      if (year > 1000)  year = year - 1900;
      if (year < 0)  year = 0;
      if (month < 0)  month = 0;
      if (month > 12)  month = 12;
      days = 0;
      if (month > 0) {
          days += 1 + dayspassed[month-1];
          if (year > 0)  days += 365*year + (year-1)/4;
          if ((year>0) && (year%4==0) && (month>2))  days++; }
      days += day;
      mydate = 1000000 + days;
      }
  else  sscanf(pd, "%lf", &mydate);
  n = sscanf(pt, "%d:%d:%d", &hour, &minute, &second);
  if (n < 3) { second=minute; minute=hour; hour=0; }
  if (n < 2) { second=minute; minute=0; }
  if (n < 1) { second=0; }
  mydate += (second + 60*(minute + 60*hour))/(24.0*3600.0);
  MsgDateHour = hour;
  MsgDateSecond = second;
  MsgDateMinute = minute;
  return mydate;
  }

char *MsgDateString(    /* converts date value into string      */
double d )              /* days after 1899-12-30'00:00:00 +1.e6 */
  {                     /* RETURN: string yy-mm-dd'hh:mm:ss     */
  static char dstr[80];
  static int dayspassed[] = {
        0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365 };
  int seconds, second, minute, hour;
  int days, day, month, year;
  int day1, day2, add;
  char sep = '.';
  if (d < -1.e-5)  return "-inf";                   //-2002-07-03
  if (d < 0)  d = 0;                                //-2002-07-03
  if (d >= 9999999)  return "+inf";
  days = floor(d);
  seconds = (d - days)*24*3600 + 0.5;
  second = seconds % 60;
  minute = (seconds/60) % 60;
  hour = seconds/3600;
  MsgDateHour = hour;
  MsgDateMinute = minute;
  MsgDateSecond = second;
  if (days == 0) {
      sprintf(dstr, "%02d:%02d:%02d", hour, minute, second);
      return dstr; }
  if (days < 1000002) {
      sprintf(dstr, "%d%c%02d:%02d:%02d", days, sep, hour, minute, second);
      return dstr; }
  days -= 1000000;
  day1 = 2;
  for (year=0; year<200; year++) {
      day2 = day1 + 365;
      if ((year) && (year%4==0)) day2++;
      if (days < day2)  break;
      day1 = day2; }
  days -= day1;
  add = ((year) && (year%4==0)) ? 1 : 0;
  for (month=1; month<=12; month++)
      if (days < dayspassed[month]+add*(month>1))  break;
  day = 1 + days - (dayspassed[month-1]+add*(month>2));
  /* if (year >= 100) */  year += 1900;                         //-2001-04-26 lj
  sprintf(dstr, "%02d-%02d-%02d%c%02d:%02d:%02d",
    year, month, day, sep, hour, minute, second);
  MsgDateYear = year;
  MsgDateMonth = month;
  MsgDateDay = day;
  return dstr;
  }

char *MsgDate( void )   // returns the current date and time
{
  static char buffer[80];
  time_t time_of_day = time(NULL);
  strftime(buffer, 79, "%Y-%m-%d %H:%M:%S", localtime(&time_of_day));
  return buffer;
}


double MsgDateSeconds( double t1, double t2 )
  {
  return (t2-t1)*24*3600;
  }

double MsgDateIncrease(   // increase date t by n seconds
double t,
int n )
{
  t += n/(24.0*3600.0);
  return t;
}

int MsgCheckPath(     // normalize path expression
char *s )             // path
{                     // RETURN: is absolute path
  char *pc, *p0;
  if (!s || !*s)  return 0;
  for (pc=s; (*pc); pc++)  if (*pc == MSG_ALTPSEP)  *pc = MSG_PATHSEP;
  if (pc[-1] == MSG_PATHSEP)  pc[-1] = 0;
  if (*s == MSG_PATHSEP)  return 1;
  if (!strchr(s, MSG_PATHSEP) && *s != '~') {   //-2004-08-20 
    p0 = strrchr(s, ':');                       // insert current directory "./"
    if (p0)  p0++;
    else     p0 = s;
    for (pc=s+strlen(s); (pc>=p0); pc--)  pc[2] = pc[0];
    p0[0] = '.';
    p0[1] = MSG_PATHSEP;
  }
  if (strchr(s, ':'))  return 1;
  return 0;
}



#ifdef  MAIN  /*#####################################################*/

int main()
  {
  char s[80], t[80];
  double d;
  for (;;) {
      fputs(":", stdout);  gets(s);
      if (!*s) break;
      trim(s);
      d = MsgDateVal(s);
      strcpy(t, MsgDateString(d));
      printf("%15.5lf  >%s<\n", d, t);
  }
  return 0;
  }
#endif
/*==========================================================================

 history:

 2001-04-27 uj 1.2.1  update with lj-version
 2001-04-28 lj 1.2.2  update
 2001-04-28 lj 1.2.3  time format (separator)
 2001-08-10 lj 1.2.4  display in vMsg()
 2001-08-15 lj 1.2.5  _MsgTokens preserves source string
 2001-09-03 lj 1.2.6  MsgUnquote()
 2001-09-07 lj 1.2.7  synchronized

===========================================================================*/
